export class Example extends HTMLElement {
  constructor() {
    super();
    const shadow = this.attachShadow({ mode: 'open' });
    shadow.innerHTML = `
      <dl>
        <div class="meta meta-value">
          <dt>Value</dt>
          <dd class="value"></dd>
        </div>
        <div class="meta meta-prop">
          <dt>Styling Hook</dt>
          <dd class="value"></dd>
        </div>
        <div class="meta meta-group">
          <dt>Shared</dt>
          <dd class="value"></dd>
        </div>
        <div class="meta meta-example">
          <dt>Example</dt>
          <dd class="example"><slot></slot></dd>
        </div>
        <div class="meta meta-description">
          <dt>Description</dt>
          <dd class="value"></dd>
        </div>
      </dl>
      <style>
        p {
          margin: 0;
        }
        dl {
          display: flex;
          flex-direction: column;
          border-bottom: 1px solid rgba(0,0,0,.15);
          padding: 1rem;
        }
        dt {
          color: rgba(0,0,0,.5);
          font-size: 0.825rem;
          font-weight: bold;
          margin-bottom: 0.125rem;
        }
        dd {
          margin: 0 0 0.5rem;
        }
        code {
          padding: 0.2em 0.4em;
          margin: 0;
          background-color: rgba(27, 31, 35, 0.05);
          border-radius: 6px;
          white-space: pre-wrap;
        }
        .example {
          box-sizing: border-box;
          border: 4px solid transparent;
        }
        :host([category="color"]) .example {
          width: 40px;
          height: 40px;
        }
        :host([category="border-radius"]) .example {
          width: 40px;
          height: 40px;
          background: #0176d3;
        }
        :host([category="shadow"]) .example {
          width: 150px;
          height: 150px;
          background: white;
        }
        :host([category="spacing"]) .example {
          width: 0;
          height: 0;
          background: #0176d3;
        }
        :host([category="sizing"]) .example {
          width: 40px;
          height: 40px;
          border-color: #0176d3;
        }
        :host([cssproperty="color"]) .example {
          background-color: currentColor;
        }
        ::slotted(a) {
          color: #0176d3;
          text-decoration: inherit;
        }
      </style>
    `;
  }

  get value() {
    return this.getAttribute('value');
  }

  get prop() {
    return this.getAttribute('prop');
  }

  get category() {
    return this.getAttribute('category');
  }

  get group() {
    return this.getAttribute('group');
  }

  get title() {
    return this.getAttribute('title');
  }

  get cssProperty() {
    const prop = this.getAttribute('cssProperty');
    if (this.category === 'spacing') {
      return `padding-top: ${this.value}; padding-left`;
    }
    return prop;
  }

  connectedCallback() {
    const propValue = this.shadowRoot.querySelector('.meta-value .value');
    const propName = this.shadowRoot.querySelector('.meta-prop .value');
    const group = this.shadowRoot.querySelector('.meta-group .value');
    const description = this.shadowRoot.querySelector('.meta-description .value');
    const example = this.shadowRoot.querySelector('.example');

    this.shadowRoot.host.attributes.forEach((el) => {
      const metaDataEl = this.shadowRoot.querySelector(`.meta-${el.name}`);
      const example = this.shadowRoot.querySelector('.meta-example');
      if (el.value === 'undefined' || el.value === 'null') metaDataEl.setAttribute('style', 'display: none;');
      if (el.name === 'value' && el.value === 'null') {
        example.setAttribute('style', 'display: none;');
      }
    });

    propValue.innerHTML = `<code>${this.value}</code>`;
    propName.innerHTML = `<code>${this.prop}</code>`;
    description.innerHTML = `<p>${this.title}</p>`;
    group.innerHTML = `<p>${this.group}</p>`;
    example.setAttribute('style', `${this.cssProperty}: ${this.value};`);
  }
}
